//+------------------------------------------------------------------+
//|                                         Synergy DynamicIndex.mq4 |
//|                                                         SwingMan |
//|                                                                  |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//|                                    Traders Dynamic Index.mq4     |
//|                                    Copyright  2006, Dean Malone |
//|                                    www.compassfx.com             |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|                                                                  |
//|                     Traders Dynamic Index                        |
//|                                                                  |
//|  This hybrid indicator is developed to assist traders in their   |
//|  ability to decipher and monitor market conditions related to    |
//|  trend direction, market strength, and market volatility.        |
//|                                                                  | 
//|  Even though comprehensive, the T.D.I. is easy to read and use.  |
//|                                                                  |
//|  Green line = RSI Price line                                     |
//|  Red line = Trade Signal line                                    |
//|  Blue lines = Volatility Band                                    | 
//|  Yellow line = Market Base Line                                  |  
//|                                                                  |
//|  Trend Direction - Immediate and Overall                         |
//|   Immediate = Green over Red...price action is moving up.        |
//|               Red over Green...price action is moving down.      |
//|                                                                  |   
//|   Overall = Yellow line trends up and down generally between the |
//|             lines 32 & 68. Watch for Yellow line to bounces off  |
//|             these lines for market reversal. Trade long when     |
//|             price is above the Yellow line, and trade short when |
//|             price is below.                                      |        
//|                                                                  |
//|  Market Strength & Volatility - Immediate and Overall            |
//|   Immediate = Green Line - Strong = Steep slope up or down.      | 
//|                            Weak = Moderate to Flat slope.        |
//|                                                                  |               
//|   Overall = Blue Lines - When expanding, market is strong and    |
//|             trending. When constricting, market is weak and      |
//|             in a range. When the Blue lines are extremely tight  |                                                       
//|             in a narrow range, expect an economic announcement   | 
//|             or other market condition to spike the market.       |
//|                                                                  |               
//|                                                                  |
//|  Entry conditions                                                |
//|   Scalping  - Long = Green over Red, Short = Red over Green      |
//|   Active - Long = Green over Red & Yellow lines                  |
//|            Short = Red over Green & Yellow lines                 |    
//|   Moderate - Long = Green over Red, Yellow, & 50 lines           |
//|              Short= Red over Green, Green below Yellow & 50 line |
//|                                                                  |
//|  Exit conditions*                                                |   
//|   Long = Green crosses below Red                                 |
//|   Short = Green crosses above Red                                |
//|   * If Green crosses either Blue lines, consider exiting when    |
//|     when the Green line crosses back over the Blue line.         |
//|                                                                  |
//|                                                                  |
//|  IMPORTANT: The default settings are well tested and proven.     |
//|             But, you can change the settings to fit your         |
//|             trading style.                                       |
//|                                                                  |
//|                                                                  |
//|  Price & Line Type settings:                           |                
//|   RSI Price settings                                             |               
//|   0 = Close price     [DEFAULT]                                  |               
//|   1 = Open price.                                                |               
//|   2 = High price.                                                |               
//|   3 = Low price.                                                 |               
//|   4 = Median price, (high+low)/2.                                |               
//|   5 = Typical price, (high+low+close)/3.                         |               
//|   6 = Weighted close price, (high+low+close+close)/4.            |               
//|                                                                  |               
//|   RSI Price Line & Signal Line Type settings                                   |               
//|   0 = Simple moving average       [DEFAULT]                      |               
//|   1 = Exponential moving average                                 |               
//|   2 = Smoothed moving average                                    |               
//|   3 = Linear weighted moving average                             |               
//|                                                                  |
//|   Good trading,                                                  |   
//|                                                                  |
//|   Dean                                                           |                              
//+------------------------------------------------------------------+

#property indicator_buffers 6
#property indicator_color1 Black
#property indicator_color2 MediumBlue
#property indicator_color3 Yellow
#property indicator_color4 MediumBlue
#property indicator_color5 Green
#property indicator_color6 Red
#property indicator_color7 DimGray
#property indicator_separate_window
#property indicator_minimum 10.0
#property indicator_maximum 90.0

//---- extern parameters --------------------------------------------+
extern int RSI_Period = 13;               //8-25
extern int RSISmooth_Period = 2;      
extern int RSIVolatilityBand_Period = 34; //20-40
extern int RSITradeSignal_Period = 7;   
extern int RSI_Price = 0;               //0-6
extern int RSI_MavgMODE = 0;            //0-3
extern int RSITradeSignal_MavgMODE = 0; //0-3
//+------------------------------------------------------------------+

#define WindowsName "Synergy DynamicIndex"
//---- oscillator levels
#define upperLineValue  68
#define medLineValue    50
#define lowerLineValue  32
#define UPPERLINE "UPPERLINE"
#define MEDLINE   "MEDLINE"
#define LOWERLINE "LOWERLINE"

//---- Buffers
double RSIBuf[],UpZone[],MdZone[],DnZone[],MaBuf[],MbBuf[];//, Line50[];

//+------------------------------------------------------------------+
//| init
//+------------------------------------------------------------------+    
int init()
  {
   IndicatorShortName(WindowsName);
   SetIndexBuffer(0,RSIBuf);
   SetIndexBuffer(1,UpZone);
   SetIndexBuffer(2,MdZone);
   SetIndexBuffer(3,DnZone);
   SetIndexBuffer(4,MaBuf);
   SetIndexBuffer(5,MbBuf);
//   SetIndexBuffer(6,Line50);   
   
   SetIndexStyle(0,DRAW_NONE); 
   SetIndexStyle(1,DRAW_LINE); 
   SetIndexStyle(2,DRAW_LINE,0,2);
   SetIndexStyle(3,DRAW_LINE);
   SetIndexStyle(4,DRAW_LINE,0,2);
   SetIndexStyle(5,DRAW_LINE,0,2);
//   SetIndexStyle(6,DRAW_LINE,0,1);
   
   SetIndexLabel(0,NULL); 
   SetIndexLabel(1,"VB High"); 
   SetIndexLabel(2,"Market Base Line"); 
   SetIndexLabel(3,"VB Low"); 
   SetIndexLabel(4,"RSI Price Line");
   SetIndexLabel(5,"Trade Signal Line");
//   SetIndexLabel(6,NULL);
   
   //SetLevelValue(0,50);
   
//   SetLevelValue(1,68);
//   SetLevelValue(2,32);
//   SetLevelStyle(STYLE_DOT,1,DimGray);
   
   SetIndexDrawBegin(0,RSIVolatilityBand_Period+RSI_Period);
   SetIndexDrawBegin(1,RSIVolatilityBand_Period+RSI_Period);
   SetIndexDrawBegin(2,RSIVolatilityBand_Period+RSI_Period);
   SetIndexDrawBegin(3,RSIVolatilityBand_Period+RSI_Period);
   SetIndexDrawBegin(4,RSIVolatilityBand_Period+RSI_Period);
   SetIndexDrawBegin(5,RSIVolatilityBand_Period+RSI_Period);
//   SetIndexDrawBegin(6,RSIVolatilityBand_Period+RSI_Period);
   return(0);
  }

//+------------------------------------------------------------------+
//| start
//+------------------------------------------------------------------+    
int start()
  {     
   DrawOscillatorLevels();
   
   double MA,RSI[];
   ArrayResize(RSI,RSIVolatilityBand_Period);
   int counted_bars=IndicatorCounted();
   int limit = Bars-counted_bars-1;
   for(int i=limit; i>=0; i--)
   {
//      Line50[i] = 50.0;
      
      RSIBuf[i] = (iRSI(NULL,0,RSI_Period,RSI_Price,i)); 
      MA = 0;
      for(int x=i; x<i+RSIVolatilityBand_Period; x++) {
         RSI[x-i] = RSIBuf[x];
         MA += RSIBuf[x]/RSIVolatilityBand_Period;
      }
      UpZone[i] = (MA + (1.6185 * StDev(RSI,RSIVolatilityBand_Period)));
      DnZone[i] = (MA - (1.6185 * StDev(RSI,RSIVolatilityBand_Period)));  
      MdZone[i] = ((UpZone[i] + DnZone[i])/2);
      }
   for (i=limit-1;i>=0;i--)  
      {
       MaBuf[i] = (iMAOnArray(RSIBuf,0,RSISmooth_Period,0,RSI_MavgMODE,i));
       MbBuf[i] = (iMAOnArray(RSIBuf,0,RSITradeSignal_Period,0,RSITradeSignal_MavgMODE,i));   
      } 
//----
   return(0);
  }

//+------------------------------------------------------------------+
//| StDev
//+------------------------------------------------------------------+    
double StDev(double& Data[], int Per)
{return(MathSqrt(Variance(Data,Per)));
}

//+------------------------------------------------------------------+
//| Variance
//+------------------------------------------------------------------+  
double Variance(double& Data[], int Per)
{double sum, ssum;
  for (int i=0; i<Per; i++)
  {sum += Data[i];
   ssum += MathPow(Data[i],2);
  }
  return((ssum*Per - sum*sum)/(Per*(Per-1)));
}

//+------------------------------------------------------------------+
//| oscillator levels / source: @cja; Traders Dynamic Index.mq4  
//+------------------------------------------------------------------+    
void DrawOscillatorLevels()
{ 
   DeleteCreateline();   
   Createline(UPPERLINE, upperLineValue, STYLE_DOT,  1,DimGray); 
   Createline(MEDLINE,   medLineValue,   STYLE_SOLID,1,DimGray);
   Createline(LOWERLINE, lowerLineValue, STYLE_DOT,  1,DimGray); 
}

void Createline(string objName, double value, int style, int width, color clr)
{   
   ObjectCreate(objName, OBJ_HLINE,WindowFind(WindowsName),0, value);      
   ObjectSet(objName, OBJPROP_STYLE, style);
   ObjectSet(objName, OBJPROP_WIDTH, width);
   ObjectSet(objName, OBJPROP_COLOR, clr);   
}

void DeleteCreateline()
{
   ObjectDelete(UPPERLINE);
   ObjectDelete(LOWERLINE);
   ObjectDelete(MEDLINE);
}